"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDepositData = exports.getCreateExtrinsic = exports.createExtrinsicEventHandler = void 0;
const xcm_config_1 = require("@moonbeam-network/xcm-config");
const util_1 = require("@polkadot/util");
const sdk_interfaces_1 = require("./sdk.interfaces");
function createExtrinsicEventHandler(config, cb) {
    return ({ events = [], status, txHash }) => {
        const hash = txHash.toHex();
        if (status.isReady) {
            cb({ status: sdk_interfaces_1.ExtrinsicStatus.Sent, txHash: hash });
        }
        if (status.isInBlock) {
            const block = status.asInBlock.toString();
            events.forEach(({ event: { data, method, section } }) => {
                if (section === config.extrinsic.pallet &&
                    method === config.extrinsic.successEvent) {
                    if (method === xcm_config_1.PolkadotXcmExtrinsicSuccessEvent.Attempted) {
                        const eventData = data.at(0);
                        if (eventData.isIncomplete) {
                            cb({
                                status: sdk_interfaces_1.ExtrinsicStatus.Failed,
                                blockHash: block,
                                txHash: hash,
                                message: eventData.asIncomplete.toHuman().join('; '),
                            });
                            return;
                        }
                    }
                    cb({
                        status: sdk_interfaces_1.ExtrinsicStatus.Success,
                        blockHash: block,
                        txHash: hash,
                    });
                }
                if (section === 'system' && method === 'ExtrinsicFailed') {
                    cb({
                        status: sdk_interfaces_1.ExtrinsicStatus.Failed,
                        blockHash: block,
                        txHash: hash,
                        message: data.join('; '),
                    });
                }
            });
        }
    };
}
exports.createExtrinsicEventHandler = createExtrinsicEventHandler;
function getCreateExtrinsic({ account, config, foreignPolkadot, primaryAccount, fee, }) {
    return (amount) => foreignPolkadot.getXcmExtrinsic(account, amount, config.extrinsic, fee, primaryAccount);
}
exports.getCreateExtrinsic = getCreateExtrinsic;
async function getDepositData({ account, asset, config, foreignPolkadot, moonChain, nativeAsset, origin, polkadot, polkadotSigner, primaryAccount, sourceAccount, }) {
    const sourceAddress = typeof sourceAccount === 'string' ? sourceAccount : sourceAccount.address;
    const meta = foreignPolkadot.getMetadata();
    const xcmFeeAssetConfig = {
        asset: config.xcmFeeAsset?.asset ?? asset,
        balance: config.xcmFeeAsset?.balance ?? config.balance,
    };
    const [assetDecimals, existentialDeposit, sourceBalance, sourceFeeBalance, sourceMinBalance = 0n, xcmFee, xcmFeeBalance, xcmFeeDecimals,] = await Promise.all([
        // assetDecimals
        polkadot.getAssetDecimals(asset),
        // existentialDeposit
        foreignPolkadot.getExistentialDeposit(),
        // sourceBalance
        foreignPolkadot.getGenericBalance(primaryAccount || sourceAddress, config.balance),
        // sourceFeeBalance
        config.sourceFeeBalance
            ? foreignPolkadot.getGenericBalance(sourceAddress, config.sourceFeeBalance)
            : undefined,
        // sourceMinBalance
        config.sourceMinBalance
            ? foreignPolkadot.getAssetMinBalance(config.sourceMinBalance)
            : undefined,
        // xcmFee
        polkadot.getAssetFee(xcmFeeAssetConfig.asset, config.source.weight, moonChain),
        // xcmFeeBalance
        foreignPolkadot.getGenericBalance(primaryAccount || sourceAddress, xcmFeeAssetConfig.balance),
        // xcmFeeDecimals
        polkadot.getAssetDecimals(xcmFeeAssetConfig.asset),
    ]);
    // Min is basically the XCM fee, if it's the same asset. If less is sent then
    // Moon* won't process the message.
    const min = config.xcmFeeAsset ? 0n : xcmFee;
    const createExtrinsic = getCreateExtrinsic({
        account,
        config,
        foreignPolkadot,
        primaryAccount,
        fee: xcmFee,
    });
    return {
        asset: { ...asset, decimals: assetDecimals },
        existentialDeposit,
        min,
        moonChainFee: {
            balance: xcmFeeBalance,
            decimals: xcmFeeAssetConfig.asset.isNative
                ? meta.decimals
                : xcmFeeDecimals,
            fee: xcmFee,
            symbol: xcmFeeAssetConfig.asset.originSymbol,
        },
        native: { ...nativeAsset, decimals: meta.decimals },
        origin,
        source: config.source,
        sourceBalance,
        sourceFeeBalance: !(0, util_1.isUndefined)(sourceFeeBalance)
            ? { ...meta, balance: sourceFeeBalance }
            : undefined,
        sourceMinBalance,
        getFee: async (amount = sourceBalance) => {
            const extrinsic = await createExtrinsic(amount);
            const info = await extrinsic.paymentInfo(sourceAccount);
            return info.partialFee.toBigInt();
        },
        send: async (amount, cb) => {
            const extrinsic = await createExtrinsic(amount);
            const hash = await extrinsic.signAndSend(sourceAccount, {
                signer: polkadotSigner,
            }, cb && createExtrinsicEventHandler(config, cb));
            return hash.toString();
        },
    };
}
exports.getDepositData = getDepositData;
//# sourceMappingURL=sdk.deposit.js.map